@@ -19,9 +19,9 @@ class SimpleRenderPipeline : public RenderPipeline {
   std::vector<std::pair<ImageF*, Rect>> PrepareBuffers(
       size_t group_id, size_t thread_id) override;
 
-  void ProcessBuffers(size_t group_id, size_t thread_id) override;
+  Status ProcessBuffers(size_t group_id, size_t thread_id) override;
 
-  void PrepareForThreadsInternal(size_t num, bool use_group_ids) override;
+  Status PrepareForThreadsInternal(size_t num, bool use_group_ids) override;
 
   // Full frame buffers. Both X and Y dimensions are padded by
   // kRenderPipelineXOffset.
