@@ -5,16 +5,10 @@
 
 #include "tools/speed_stats.h"
 
-#ifndef __STDC_FORMAT_MACROS
-#define __STDC_FORMAT_MACROS
-#endif
-
-#include <inttypes.h>
-#include <math.h>
-#include <stddef.h>
-#include <stdio.h>
-
 #include <algorithm>
+#include <cmath>
+#include <cstddef>
+#include <cstdio>
 #include <string>
 
 namespace jpegxl {
@@ -58,12 +52,12 @@ bool SpeedStats::GetSummary(SpeedStats::Summary* s) {
     s->central_tendency = pow(product, 1.0 / (elapsed_.size() - 1));
     s->variability = 0.0;
     s->type = " geomean:";
-    if (isnormal(s->central_tendency)) return true;
+    if (std::isnormal(s->central_tendency)) return true;
   }
 
   // Else: median
   std::sort(elapsed_.begin(), elapsed_.end());
-  s->central_tendency = elapsed_.data()[elapsed_.size() / 2];
+  s->central_tendency = elapsed_[elapsed_.size() / 2];
   double stdev = 0;
   for (size_t i = 0; i < elapsed_.size(); i++) {
     double diff = elapsed_[i] - s->central_tendency;
@@ -92,7 +86,7 @@ std::string SummaryStat(double value, const char* unit,
     snprintf(variability, sizeof(variability), " (stdev %.3f)", stdev);
   }
 
-  snprintf(stat_str, sizeof(stat_str), ",%s %.3f %s/s [%.2f, %.2f]%s", s.type,
+  snprintf(stat_str, sizeof(stat_str), "%s %.3f %s/s [%.2f, %.2f]%s", s.type,
            value_tendency, unit, value_min, value_max, variability);
   return stat_str;
 }
@@ -107,13 +101,10 @@ bool SpeedStats::Print(size_t worker_threads) {
   std::string mps_stats = SummaryStat(xsize_ * ysize_ * 1e-6, "MP", s);
   std::string mbs_stats = SummaryStat(file_size_ * 1e-6, "MB", s);
 
-  fprintf(stderr,
-          "%" PRIu64 " x %" PRIu64 "%s%s, %" PRIu64 " reps, %" PRIu64
-          " threads.\n",
-          static_cast<uint64_t>(xsize_), static_cast<uint64_t>(ysize_),
-          mps_stats.c_str(), mbs_stats.c_str(),
-          static_cast<uint64_t>(elapsed_.size()),
-          static_cast<uint64_t>(worker_threads));
+  fprintf(stderr, "%d x %d, %s, %s, %d reps, %d threads.\n",
+          static_cast<int>(xsize_), static_cast<int>(ysize_), mps_stats.c_str(),
+          mbs_stats.c_str(), static_cast<int>(elapsed_.size()),
+          static_cast<int>(worker_threads));
   return true;
 }
 
